/****************************************************************************
** Diese DLL ist fr MegaCAD evo.II geschrieben. 
** Die DLL speichert alle Layer der aktuell geffneten Zeichnung getrennt 
** nach Layern, unter angabe des aktuellen Zeichungsnamen, der Layernummer 
** und ihres Layernamens in das aktuelle Verzeichnis.
** Leider funktioniert das speichern nur 2D, da MegaCAD nur eine 2D Speicherung
** einzelner Elemente von aussen zugnglich macht.
**
** Created: Sat Sep 27 11:45:00 2003
** 
** by Karsten Hiller 
** 
****************************************************************************/

#include "stdafx.h"
#include "qapplication.h"
#include <qarray.h>
#include <qstring.h>
#include <qmessagebox.h> 
#include <lg_error.h>

//Erweiterung der Zeichenconstanten fr Typumwandlung
#define GT_NO_TRANSLATE	0	  // keine Zeichenumwandlung
#define GT_OEM_TO_ANSI -1	  // OEM nach ANSI 
#define GT_ANSI_TO_OEM	1	  // ANSI nach OEM

#define ELEMENT_RANGE 1000	  // Anfangsgrsse und Schrittweite fr Element Array
#define NAME_MAXLEN 255		  // Lnge der Dateinamen 255 o. 512 Zeichen (WIN9x/WIN2K)

//globale Variablen, knnten auch als struct SelElmLay bergeben werden,
//aber so ist das ganze fr unerfahrene nachvollziebarer
QArray<ulong> elmList(ELEMENT_RANGE); // Elementliste 
ulong		  elmPos = 0;			  // Zhler fr die akt. Position in elmList
uchar		  elmData[32000];		  // BytePuffer fr variable ElementDaten  
ushort		  elmLayNum = 0;		  // LayerNummer der auszuwhlenden Elemente

/*****************************************************************************
** Hilfe Fenster				  	
*****************************************************************************/
short HelpWindow(void *parg)
{
  QString text("sprtalay.dll - dient dazu alle in einer Zeichnung vorhanden Layer ");
		text+=("getrennt nach ihren Namen zu speichern.\n\n");
		text+=("Die Speicherung erfolgt dabei unter Verwendung des aktuellen Pfades ");
		text+=("und Zeichnungsnamens, welche in einem Dialog gendert werden knnen.\n");
		text+=("Zwecks Unterscheidung der unterschiedlichen Layer wird an den Zeichnungsnamen ");
		text+=("jeweils getrennt durch einen Unterstich die Layernummer und \n");
		text+=("der Layername angehngt. Leere Layer d.h. Layer welche zwar einen Namen besitzen, ");
		text+=("aber keine Elemente werden nicht gespeichert. \n");
		text+=("Die Angabe der Layernummer erfolgt aus Sicherheitsgrnden, damit gleichnamige ");
		text+=("Layer nicht einfach berschrieben werden.\n");
		text+=("Gruppenzugehrigkeit der Elementdaten bleibt erhalten.\n");
		text+=("Leider knnen alle Daten nur in 2D gespeichert werden, da MegaCAD nur eine ");
		text+=("2D-Funktion zum speichern einzelner Elementdaten bereitstellt.\n\n ");
		text+=("by K.Hiller (2003.09.27)\n\n");
		text+=("This is FREEWARE and used Qt from Trolltech");
  QMessageBox::about( NULL, "sprtalay.dll help window", text ); 
  QMessageBox::aboutQt( NULL, QString::null ); 
  return( TRUE );
}

/*****************************************************************************
** Selektiert alle Elemente eines bestimmten Layers und					  	
** trgt dessen Element ID in ein dynamisches Array ein						
** dessen Grsse ggf. vergrssert wird.										
*****************************************************************************/
short SelElmLay( void *parg )
{
  t_entity *ent;				  //Zeiger auf Elementstrucktur

  ent = (t_entity*)parg;
  
  if( elmPos == elmList.size() )  //Teste ob Speicherplatz in elmList noch reicht
  {						
	//vergrssert die elmListe um element_range und gibt eine Fehlermeldung aus
	//falls dies nicht mglich war und bricht die ElementSchleife 'SelectLoop' ab
	if( elmList.resize( elmPos + ELEMENT_RANGE ) == 0) 
	{	  
	  QMessageBox::warning( 0, "SelElmLay", 
							"Es konnte kein zustzlicher Speicher reserviert werden."
							"Es werden Daten fehlen!" );
	  return( BREAK_LOOP );			//zurck zur ElementSchleife und Abbruch d. Schleife
	}
  }
  if( ent->attr.lay == elmLayNum )	//Vergleiche Layernummern
  {
	elmList.at(elmPos) = ent->id;	//speichert die Elementnummer in elmListe
	elmPos++;						//erhht Zhler de elmListe um 1
  }
  return( CONTINUE_LOOP );			//zurck zur ElementSchleife ohne Abbruch
}

/*****************************************************************************
** Macht aus dem MegaCAD Spass was lesbares	und verarbeitbares			  	
** also wandelt den die Zeichen Byte (uchar) in Char um 
** und ndert ggf. den Zeichensatz ,,,... in was lesbares um					
*****************************************************************************/
short ByteToChar( char *dName, uchar *sName, ushort oemAnsi )
{
  ushort  i = 0;

  do  //solange bis StringEnde oder FeldEnde erreicht ist
  {
	dName[i] = (char) sName[i];	  //Umwandlung von byte (uchar) nach char
	i++;						  //Zhler = Zhler +1
  }
  while( dName[i] != '\0' || i <= strlen(dName) );
  
  switch( oemAnsi )	//Zeichensatzumwandlung von OEM <--> Char (WinAPI-Funktion)
  {
	case GT_OEM_TO_ANSI :  OemToChar(dName, dName); break;
	case GT_ANSI_TO_OEM :  CharToOem(dName, dName); break;
  }
  return( TRUE );
}


/*****************************************************************************
**
** Haupprogamm
** mit Schleife ber alle Layer und Speicherung der ElementDaten als 2D-PRT
**
*****************************************************************************/
short MegaMain( char *file,
				char *args )
{
  h_LGList	  hLayLGList	= NULL;				//Handle der Layerliste
  h_LGItem	  layItem		= NULL;				//Handle des LayerItem
  ulong		  layAnzahl		= 0;				//Menge der Layer	
  ushort	  layErr		= 0;				//ErrorCodeVariable
  t_LayGrpDta layData;							//Struktur der Layerdaten
  char		  layName[LAYGRP_MAXLEN];			//Name des Layers
  uchar		  currentName[NAME_MAXLEN]	= "\0";	//Name der aktuellen Zeichung
  char		  saveName[NAME_MAXLEN]		= "\0";	//Name der zuspeichernden Datei
  char		  saveNameLay[NAME_MAXLEN]	= "\0";	//Name der zuspeichernden Datei + Layername
  
  t_entity ent;						//GebildeStrucktur
  t_mat44  ecs;						//3D-MatrixStrucktur
  
  ulong		filter1 = 0xffffffff;	// Selektionskriterium -> (2D) alle Elemente erlaubt
  ulong		filter2 = 0xffffffff;	// Selektionskriterium -> (3D) alle Elemente erlaubt
  
  int argc = 2;	  // 2 Parameter
  char *argv[2];  // Feld fr Parameter

  argv[0] = file; // Parameter 1 - Filename
  argv[1] = args; // Parameter 2 - Liste der Argumente
  
  QApplication a( argc, argv );	// Application starten (fr Messages)
  
  // Text in der Statuszeile von MegaCAD
  SetFuncText( "Speicherung der aktuellen Zeichnung getrennt nach Layern" );
  // Hilfe Funktion der F1-Taste zuweisen (callback function)
  SetHelpFunc( HelpWindow );              

  //Name der geladenen Zeichnung ermitteln 
  //wenn eine Zeichnung geladen ist diese Layerweise Speichern
  if( 0 != GetCurrentFile( currentName ) )		
  {												
	//umwandeln der Zeichen GT_OEM_TO_ANSI GT_ANSI_TO_OEM GT_NO_TRANSLATE
	ByteToChar( saveName, currentName, GT_NO_TRANSLATE);	
	
	// let the user select a filename, if OK then then do it
	if( TRUE == GetFilename( "Speichern aller Layer in einzelnen Dateien", saveName ) )
	{
	  //Filename am Punkt abschneiden (.prt) und NULL anhngen
	  strcat( strtok( saveName, "." ), "\0" );			
	
	  //erzeuge Kopie/Handle der Layerliste
	  hLayLGList = LG_CreateList(LGL_LAYER, &layErr);	
	  LG_Error("LGL-LAYER-CREATE", &layErr);			//Fehlerbehandlung mit Message
	
	  //ermittelt die Anzahl der Layer
	  layAnzahl = LG_GetInfo(hLayLGList, LGINF_GETCOUNT, &layErr);
	  LG_Error("LGL-LAYER-GETINFO", &layErr);			//Fehlerbehandlung mit Message
	
	  layItem = LGITEM_ROOT;							//setze auf ROOT-item (Anfang)
	  for( ushort i=0; i<=layAnzahl; i++)				//Schleife ber alle Layer
	  {
		//holt immer den linear nchsten Layer aus der LayerStrucktur
		layItem = LG_ItemOp(hLayLGList, layItem, 0, &layData, LGOP_LINEARNEXT, &layErr);
		//Fehlerbehandlung mit Message, bei Fehler Schleife abbrechen
		if( 2 < LG_Error("LGL-LAYER-ITEM", &layErr) ) break;	
	  
		elmLayNum = layData.num;				 //Layernummer ermitteln
	  
//		if( 1 != QMessageBox::information( 0, "Test-Message",
//										   QString( "LayerItem %1" ).arg( elmLayNum),
//										   "Abbruch", "OK", "", 1, -1 ) ) break;

		elmPos = 0;							 //Elementzhler auf Null setzen
		// Schleife ber alle Elemente 
		EntityLoop( filter1,filter2,SelElmLay,
	  				&ent,&elmData,&ecs,sizeof( elmData ) );
	
		if( elmPos != 0 )	//teste ob berhaupt Element im Layer wahren 
		{
		  //umwandeln des Zeichensatzes (GT_OEM_TO_ANSI / GT_NO_TRANSLATE / GT_ANSI_TO_OEM)
		  ByteToChar( layName, layData.txt, GT_NO_TRANSLATE); 
		  //Speichernamen zusammenstellen (Zeichungsnamen_Layernummer_Layername.prt)
		  sprintf( saveNameLay, "%s_%u_%s.prt\0", saveName, elmLayNum, layName);
		  // save entities to 2D.PRT, Elemente des Layers speichern
		  // bei einem Fehler Message anzeigen
		  if( FALSE == SaveToPrt( saveNameLay, elmList.data(), elmPos, 0L) )
		  {
			QMessageBox::information( 0, "Save-Message",
									  "Not saved" );
		  }
		}
	  }
	}
  }
  else
  {
    QMessageBox::information( 0, "SelElmLay",
							  "Keine Zeichung geladen. \n"
							  "Es werden keine Daten gespeichert!" );
  } 
  LG_Replace(hLayLGList, NULL, -1);	  //(-1)Handles freigeben
  a.exit(0);						  //beendet die Application
  return TRUE;
}

